#!/usr/bin/env python
"""A simple script which takes stdin and writes it to a logfile using the WatchedFileHandler so that
stdout/err of programs can be captured while also allowing log rotation of these files.

Usage:
  log.py -h | --help
  log.py <filename> [--format=<format>] [--datefmt=<damefmt>] [--tee]
  log.py --tee [--format=<format>] [--datefmt=<damefmt>]

Options:
  -h --help              Help.
  -f --format=<format>   Logging format to use. [Default: %(message)s]
  -d --datefmt=<damefmt> Date format to use. [Default: %Y-%m-%d %H:%M:%S].
  -t --tee               Write input to stdout as well as the file.

'datefmt' Won't take effect unless 'format' is given with a date identifier (like %(asctime)s)
 in it.
"""

from docopt import docopt
import logging
import logging.handlers
import os
import sys


def main():
    """Main"""
    args = docopt(__doc__)
    tee = args["--tee"]
    formatter = logging.Formatter(
        args.get("--format", "%(message)s"),
        datefmt=args.get("--datefmt", "%Y-%m-%d %H:%M:%S"),
    )
    log = logging.getLogger(__name__)
    if args["<filename>"]:
        handler = logging.handlers.WatchedFileHandler(args["<filename>"], "a", "utf-8")
        handler.setLevel(logging.INFO)
        handler.setFormatter(formatter)
        log.addHandler(handler)
    log.setLevel(logging.INFO)
    if tee:
        if hasattr(sys.stdout, "fileno"):
            # Force stdout to be line-buffered
            sys.stdout = os.fdopen(sys.stdout.fileno(), "w", 1)
        if hasattr(sys.stderr, "fileno"):
            # Force stderr to be line-buffered
            sys.stderr = os.fdopen(sys.stderr.fileno(), "w", 1)
        streamhandler = logging.StreamHandler()
        streamhandler.setLevel(logging.INFO)
        streamhandler.setFormatter(formatter)
        log.addHandler(streamhandler)
    for line in iter(sys.stdin.readline, ""):
        log.info(line.rstrip().replace("%", "%%"))


if __name__ == "__main__":
    main()



import os
import sys
import subprocess

from colorama import Fore, Style

from reversefold.util import multiproc


def escape_double_quotes(val):
    return val.replace('"', '''"'"'"''')


def escape_single_quotes(val):
    return val.replace("'", """'"'"'""")


class SSHException(Exception):
    pass


class SSHHost(object):
    # Used to know when no value is passed into stdin as None is a valid value
    __SENTINEL = object()

    def __init__(
        self,
        host,
        port=None,
        user=None,
        output_prefix="",
        prefix_pad_length=28,
        connect_timeout=5,
        # Default to no specific cipher to allow the client and server to negotiate one.
        # Can be used to force a specific cipher.
        cipher=None,
        # Default to not checking host keys as cloud servers make it super annoying.
        # Also default log level to ERROR so we don't see output about the host keys.
        check_host_keys=False,
        ssh_log_level="ERROR",
        identity=None,
        use_color=True,
        quiet=True,
    ):
        self.host = host
        self.port = port
        self.user = user
        self.prefix = output_prefix
        self.connect_timeout = connect_timeout
        self.cipher = cipher
        self.check_host_keys = check_host_keys
        self.ssh_log_level = ssh_log_level
        if identity is None:
            self.identity = os.environ.get("IDENTITY", None)
        else:
            self.identity = identity
        self.use_color = use_color
        self.quiet = quiet
        self.host_prefix = "%s[%s%s%s%s%s%s%s]%s" % (
            Style.BRIGHT if self.use_color else "",
            Style.NORMAL if self.use_color else "",
            Fore.LIGHTBLUE_EX if self.use_color else "",
            "%s@" % (user,) if user is not None else "",
            host,
            ":%s" % (port,) if port is not None else "",
            Fore.RESET if self.use_color else "",
            Style.BRIGHT if self.use_color else "",
            Style.NORMAL if self.use_color else "",
        )
        text_len = (
            len(self.host_prefix)
            - len(Style.BRIGHT) * 2
            - len(Style.NORMAL) * 2
            - len(Fore.LIGHTBLUE_EX)
            - len(Fore.RESET)
        )
        if text_len < prefix_pad_length:
            self.host_prefix += " " * (prefix_pad_length - text_len)
        self.full_prefix = "%s%s" % (self.prefix, self.host_prefix)

    def puts(self, line=None):
        """
        Write line to stdout prefixed by host
        """
        if line is None:
            line = ""
        elif not isinstance(line, str):
            line = line.decode(errors="backslashreplace")

        sys.stdout.write("%s %s" % (self.full_prefix, line))
        sys.stdout.flush()

    def _get_ssh_options(self, force_tty=False):
        ssh_options = [
            # compress
            "-C",
            # Don't even try to ask for a password if key auth fails
            "-o",
            "BatchMode=yes",
            # Send a null packet every 5 seconds to make sure our connection stays
            # open as long as needed.
            "-o",
            "ServerAliveInterval=5",
            # Explicitly set ControlMaster and ControlPath to make sure that scripted SSH sessions don't accidentally
            # clobber each other. (Imagine one SSH to a host starts, then a second one starts and the first ssh is
            # stopped by the script before the second is finished. The first SSH session being terminated will terminate
            # the second while it may still be running a command.)
            "-o",
            "ControlMaster=no",
            "-o",
            "ControlPath=none",
        ]

        if not self.check_host_keys:
            ssh_options.extend(
                ["-o", "UserKnownHostsFile=/dev/null", "-o", "StrictHostKeyChecking=no"]
            )

        if self.ssh_log_level is not None:
            ssh_options.extend(["-o", "LogLevel=%s" % (self.ssh_log_level,)])

        if self.cipher is not None:
            ssh_options.extend(["-c", str(self.cipher)])

        if self.port is not None:
            ssh_options.extend(["-p", str(self.port)])

        if self.user is not None:
            ssh_options.extend(["-l", str(self.user)])

        if self.connect_timeout is not None:
            ssh_options.extend(["-o", "ConnectTimeout=%s" % (self.connect_timeout,)])

        if force_tty:
            ssh_options.extend(["-t", "-t"])

        if self.identity is not None:
            ssh_options.extend(["-o", "IdentityFile=%s" % (self.identity,)])

        if self.quiet:
            ssh_options.append("-q")

        return ssh_options

    def _get_ssh_cmd(self, force_tty=False):
        """Get the array needed to run subprocess.Popen with ssh"""

        sshcmd = ["ssh"]
        sshcmd.extend(self._get_ssh_options(force_tty))
        sshcmd.append(str(self.host))

        return sshcmd

    def _start(
        self,
        executable,
        command,
        cwd=None,
        output_running=False,
        stdin=__SENTINEL,
        close_stdin=True,
        force_tty=False,
    ):
        cmds = []
        if cwd is not None:
            cmds.append("cd '%s'" % (escape_single_quotes(cwd),))

        cmds.append(command)

        run_cmd = "; ".join(cmds)

        if output_running:
            self.puts(run_cmd)

        sshcmd = self._get_ssh_cmd(force_tty)
        sshcmd.append("%s '%s'" % (executable, escape_single_quotes(run_cmd)))

        stdin_type = (
            subprocess.PIPE
            if stdin is SSHHost.__SENTINEL or isinstance(stdin, str)
            else stdin
        )

        ssh = subprocess.Popen(
            sshcmd, stdin=stdin_type, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

        if stdin_type == subprocess.PIPE:
            if isinstance(stdin, str):
                ssh.stdin.write(stdin)
            if close_stdin:
                ssh.stdin.close()
        return ssh

    def start(
        self,
        command,
        cwd=None,
        output_running=False,
        stdin=__SENTINEL,
        close_stdin=True,
        force_tty=False,
        capture_output=False,
    ):
        proc = self._start(
            "/bin/bash -c", command, cwd, output_running, stdin, close_stdin, force_tty
        )
        (stdout, stderr, threads) = multiproc.run_subproc(
            proc,
            output_func=self.puts,
            wait=False,
            capture_output=capture_output,
            use_color=self.use_color,
        )
        return (proc, threads, stdout, stderr)

    def _run(
        self,
        executable,
        command,
        cwd=None,
        output_running=False,
        stdin=__SENTINEL,
        capture_output=False,
    ):
        """
        Runs command via ssh and executable.
        """
        ssh = self._start(executable, command, cwd, output_running, stdin)
        (stdout, stderr) = multiproc.run_subproc(
            ssh,
            output_func=self.puts,
            capture_output=capture_output,
            use_color=self.use_color,
        )

        if ssh.returncode:
            raise SSHException("ssh return code was %r" % (ssh.returncode,))

        return (stdout, stderr)

    def run(
        self,
        command,
        cwd=None,
        output_running=False,
        stdin=__SENTINEL,
        capture_output=True,
    ):
        return self._run(
            "/bin/bash -c", command, cwd, output_running, stdin, capture_output
        )

    def sudo(
        self,
        command,
        cwd=None,
        output_running=False,
        stdin=__SENTINEL,
        capture_output=True,
    ):
        return self._run(
            "sudo /bin/bash -c", command, cwd, output_running, stdin, capture_output
        )



from datetime import datetime
import logging
import os
import sys


LOG = logging.getLogger(__name__)


RATE_LIMIT_SENTINEL = object()


def rate_limit_gen(gen, period, limit):
    start = datetime.now()
    # start at -1 since we pre-increment below and we actually want to allow up to `limit` values to be generated
    # within `period` before we yield RATE_LIMIT_SENTINEL.
    count = -1
    for value in gen:
        count += 1
        if datetime.now() - start < period:
            if count == limit:
                yield RATE_LIMIT_SENTINEL
                continue
            elif count > limit:
                continue
        else:
            start = datetime.now()
            count = 0
        yield value


def chunked(seq, chunk_size):
    """Take an input sequence (any iterable, including generators) and yield it in chunks of the given size.
        [i for i in chunked(xrange(1, 10), 2)]
        [[1, 2], [3, 4], [5, 6], [7, 8], [9]]
    """
    chunk = []
    i = 0
    for val in seq:
        chunk.append(val)
        i += 1
        if i == chunk_size:
            yield chunk
            chunk = []
            i = 0
    if i > 0:
        yield chunk


def force_stdout_line_buffer():
    if hasattr(sys.stdout, "fileno"):
        # Force stdout to be line-buffered
        sys.stdout = os.fdopen(sys.stdout.fileno(), "w", 1)
    else:
        LOG.warning("sys.stdout has no fileno attribute, it cannot be line-buffered")



#!/usr/bin/env python
"""Connects to a MySQL server and runs a simple query to make sure it is still responding
about every second.

Usage:
  mysqlping.py -h | --help
  mysqlping.py <hostname> [--port=<port>] [--username=<username>] [--password=<password>]

Options:
  -h --help                 Help.
  <hostname>                The hostname of the MySQL server to "ping".
  -P <port> --port=<port>   Port to connect to. [Default: 3306]
  -u <username> --user=<username> --username=<username> Username to authenticate as.
  -p <password> --pass=<password> --password=<password> Password to authenticate with.
"""

import datetime
from docopt import docopt
import MySQLdb
import socket
import sys
import time

i = 0
down_start = None


def log(msg):
    global i
    print("%s [%r] %s" % (datetime.datetime.now(), i, msg))
    sys.stdout.flush()
    i += 1


def mysqlping(hostname, port=3306, username=None, password=None):
    global down_start
    socket.setdefaulttimeout(1)
    while True:
        try:
            if username is None:
                conn = MySQLdb.connect(hostname, port=port, connect_timeout=1)
            else:
                conn = MySQLdb.connect(
                    hostname,
                    port=port,
                    user=username,
                    passwd=password,
                    connect_timeout=1,
                )
            if down_start is not None:
                log("Connected, downtime %s" % (datetime.datetime.now() - down_start,))
                down_start = None
            else:
                log("Connected")
            time.sleep(1)
            while True:
                cur = conn.cursor()
                cur.execute("SELECT NOW()")
                log("Ping %r" % (cur.fetchone(),))
                time.sleep(1)
        except Exception as e:
            if down_start is None:
                down_start = datetime.datetime.now()
            log("Down for %s so far: %s" % (datetime.datetime.now() - down_start, e))
            time.sleep(1)


if __name__ == "__main__":
    args = docopt(__doc__)
    mysqlping(
        args["<hostname>"], int(args["--port"]), args["--username"], args["--password"]
    )



import os
import time


class FileLineFollower(object):
    def __init__(self, file):
        self.file = file
        self.finish = False

    def __iter__(self):
        buf = []
        while True:
            new_line = self.file.readline()
            if not new_line:
                if self.finish:
                    raise StopIteration()
                time.sleep(0.1)
                continue
            buf.append(new_line)
            if buf[-1][-1] == "\n":
                # Don't include the trailing newline in the output
                yield "".join(buf)[:-1]
                buf = []


class FilenameLineFollower(FileLineFollower):
    """
    Implements a simple (naive) file streamer which will emit lines
    until the finish property is set to False.

    Ex:
    with Follower('a') as f:
        for line in f:
            print line
            if line == 'a':
                f.finish = True
    """

    def __init__(self, filename, tail_only=False):
        # Explicitly setting file to None as we set it in __enter__
        super(FilenameLineFollower, self).__init__(file=None)
        self.filename = filename
        self.tail_only = tail_only

    def __enter__(self):
        self.file = open(self.filename, "r")
        if self.tail_only:
            # Go to the end of the file
            self.file.seek(0, 2)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.file.close()
        self.file = None


LineFollower = FilenameLineFollower


class FDFollower(object):
    def __init__(self, fd, chunk_size):
        self.fd = fd
        self.chunk_size = chunk_size
        self.finish = False

    def __iter__(self):
        while True:
            data = os.read(self.fd, self.chunk_size)
            if not data:
                if self.finish:
                    raise StopIteration()
                time.sleep(0.1)
                continue
            yield data


class FilenameFollower(FDFollower):
    """
    Implements a simple (naive) file streamer which will emit bytes
    until the finish property is set to False.

    Ex:
    with Follower('a') as f:
        i = 0
        for data in f:
            sys.stdout.write(data)
            i += 1
            if i == 50:
                f.finish = True
    """

    def __init__(self, filename, tail_only=False, chunk_size=1024):
        # Setting fd to None as we set it in __enter__
        super(FilenameFollower, self).__init__(fd=None, chunk_size=chunk_size)
        self.filename = filename
        self.tail_only = tail_only

    def __enter__(self):
        self.fd = os.open(self.filename, os.O_RDONLY | os.O_NONBLOCK)
        if self.tail_only:
            # Go to the end of the file
            os.lseek(self.fd, 0, os.SEEK_END)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        os.close(self.fd)


Follower = FilenameFollower



import threading
import sys

from colorama import Fore, Style


DEFAULT_OUT_PREFIX = "%s[%s%sout%s%s]%s " % (
    Style.BRIGHT,
    Style.NORMAL,
    Fore.GREEN,
    Fore.RESET,
    Style.BRIGHT,
    Style.NORMAL,
)
DEFAULT_OUT_PREFIX_PLAIN = "[out] "

DEFAULT_OUT_POSTFIX = ""
DEFAULT_OUT_POSTFIX_PLAIN = ""

DEFAULT_ERR_PREFIX = "%s[%s%serr%s%s]%s%s " % (
    Style.BRIGHT,
    Style.NORMAL,
    Fore.RED,
    Fore.RESET,
    Style.BRIGHT,
    Style.NORMAL,
    Fore.RED,
)
DEFAULT_ERR_PREFIX_PLAIN = "[err] "

DEFAULT_ERR_POSTFIX = Fore.RESET
DEFAULT_ERR_POSTFIX_PLAIN = ""


class Pipe(object):
    def __init__(self, input_stream, output_func):
        self.input_stream = input_stream
        self.output_func = output_func

    def flow(self):
        while True:
            data = self.input_stream.read(65535)
            if not data:
                break
            self.output_func(data)


class LinePipe(object):
    # TODO: input and output_func are not symmetric, perhaps they should
    # use the same interface?
    def __init__(self, prefix, input_stream, buf, output_func, postfix, capture_output):
        self.prefix = prefix
        self.input_stream = input_stream
        self.buf = buf
        self.output_func = output_func
        self.postfix = postfix
        self.capture_output = capture_output

    def flow(self):
        for line in self.input_stream:
            try:
                line = line.decode(errors="backslashreplace")
            except AttributeError:
                pass
            if self.capture_output:
                self.buf.append(line)
            self.output_func("%s%s%s" % (self.prefix, line.rstrip(), self.postfix))


# TODO: Remove output capture entirely and allow the caller's output_func to handle it
# TODO: Remove use_color and let the caller just specify non-colored versions
def run_subproc(
    proc,
    prefix="",
    wait=True,
    output_func=None,
    out_prefix=None,
    out_postfix=None,
    err_prefix=None,
    err_postfix=None,
    capture_output=True,
    use_color=True,
):
    """
    Runs a single subprocess, outputting then returning the stderr and stdout

    returns: stdout, stderr
    """
    if out_prefix is None:
        out_prefix = DEFAULT_OUT_PREFIX if use_color else DEFAULT_OUT_PREFIX_PLAIN
    if out_postfix is None:
        out_postfix = DEFAULT_OUT_POSTFIX if use_color else DEFAULT_OUT_POSTFIX_PLAIN
    if err_prefix is None:
        err_prefix = DEFAULT_ERR_PREFIX if use_color else DEFAULT_ERR_PREFIX_PLAIN
    if err_postfix is None:
        err_postfix = DEFAULT_ERR_POSTFIX if use_color else DEFAULT_ERR_POSTFIX_PLAIN
    if output_func is None:
        output_func = sys.stdout.write
    stdout = []
    stderr = []
    threads = []
    try:
        for (prefix, pipe, buf, postfix) in [
            ("%s%s" % (prefix, err_prefix), proc.stderr, stderr, err_postfix),
            ("%s%s" % (prefix, out_prefix), proc.stdout, stdout, out_postfix),
        ]:
            if pipe is None or pipe.closed:
                continue
            thread = threading.Thread(
                name=prefix,
                target=LinePipe(
                    prefix, pipe, buf, output_func, postfix, capture_output
                ).flow,
            )
            threads.append(thread)
            thread.start()
        if wait and proc.returncode is None:
            proc.wait()
        if wait:
            return (stdout, stderr)
        return (stdout, stderr, threads)
    finally:
        if wait:
            # by this time the proc should be finished and all pipes should be closed
            # but just in case...
            terminate_subproc(proc, threads)


def terminate_subproc(proc, threads):
    for thread in threads:
        thread.join()
    for pipe in [proc.stdin, proc.stderr, proc.stdout]:
        if pipe is None or pipe.closed:
            continue
        pipe.close()
    try:
        proc.terminate()
    except Exception:
        pass
    try:
        proc.wait()
    except Exception:
        pass



#!/usr/bin/env python
"""Watches files for new lines and prints them prefixed with the filename.

Usage:
  tail.py -h | --help
  tail.py [--no-force-line-buffer]
          [--rate-limit=<count>] [--rate-period=<seconds>]
          [--each-rate-limit=<count>] [--each-rate-period=<seconds>]
          <filename>...

Options:
  -h --help                                  Help.
  --no-force-line-buffer                     Don't force stdout to be line-buffered.
  -l <count> --rate-limit=<count>            A limit to the number of lines to be output within rate-period
  -p <seconds> --rate-period=<seconds>       The period in seconds that the rate-limit is applied to.
  -L <count> --each-rate-limit=<count>       A limit to the number of lines to be output within rate-period for each individual file
  -P <seconds> --each-rate-period=<seconds>  The period in seconds that the rate-limit is applied to for each individual file.
  <filename>...                              A list of filenames to tail. A single dash ("-") can be used to indicate reading from stdin instead of a filename.

If more than `rate-limit` lines are received within `rate-period` seconds then a single line of "..." will be output and
all subsequent lines received within that period will be ignored.

Each of the rate options supports an empty value to disable the rate limiting.
"""
from __future__ import print_function
from datetime import timedelta
import os

from queue import Empty, Queue
import sys
import threading
import time

from docopt import docopt

import reversefold.util.follow
from reversefold.util import rate_limit_gen, RATE_LIMIT_SENTINEL

# TODO: Add option to output partial lines after 1s of being buffered.
#       Keep partial line in buffer to output with more data later.
# TODO: Handle file truncation
# TODO: Handle file disappearing
# TODO: Handle file appearing


class LineQueue(object):
    def __init__(self, stop, rate_limit=None, rate_period=None):
        self.queue = Queue()
        self.stop = stop
        self.rate_limit = rate_limit
        self.rate_period = (
            None if rate_period is None else timedelta(seconds=rate_period)
        )

    def _line_gen(self):
        while not self.stop.is_set():
            try:
                yield self.queue.get(block=True, timeout=0.1)
            except Empty:
                pass

    def get_line_gen(self):
        gen = self._line_gen()
        if self.rate_limit is None or self.rate_period is None:
            return gen
        return rate_limit_gen(gen, self.rate_period, self.rate_limit)

    def handle_line(self, line):
        self.queue.put(line)


class QueueChain(object):
    def __init__(self, in_queue, out_queue, rate_limit_line=None):
        self.in_queue = in_queue
        self.out_queue = out_queue
        self.rate_limit_line = rate_limit_line

    def run(self):
        # Note: No need of a stop event here as the in_queue will stop iterating when its stop event is set
        for line in self.in_queue.get_line_gen():
            if line is RATE_LIMIT_SENTINEL:
                if self.rate_limit_line is None:
                    continue
                line = self.rate_limit_line
            self.out_queue.handle_line(line)


class Master(object):
    def __init__(self, stop, rate_limit=None, rate_period=None):
        self.line_queue = LineQueue(stop, rate_limit, rate_period)

    def run(self):
        for line in self.line_queue.get_line_gen():
            if line is RATE_LIMIT_SENTINEL:
                line = "..."
            print(line)


try:
    from watchdog.events import FileSystemEventHandler
    from watchdog.observers import Observer

    class TailHandler(FileSystemEventHandler):
        def __init__(self, filename, thefile, line_queue, prefix=""):
            super(TailHandler, self).__init__()
            self.prefix = prefix
            self.filename = filename
            self.thefile = thefile
            self.line = ""
            self.line_queue = line_queue

        def on_modified(self, event):
            if event.src_path != self.filename:
                return
            while True:
                new_line = self.thefile.readline()
                if not new_line:
                    return
                self.line += new_line
                if self.line[-1] == "\n":
                    self.line_queue.handle_line(self.prefix + self.line[:-1])
                    self.line = ""


except ImportError:
    TailHandler = None  # type: ignore


def tail_file(file, line_queue, prefix=""):
    follower = reversefold.util.follow.FileLineFollower(file)
    for line in follower:
        line_queue.handle_line(prefix + line)


def tail_filename(filename, line_queue, prefix=""):
    if not os.path.exists(filename):
        sys.stderr.write("file %s does not exist\n" % (filename,))
        return

    with reversefold.util.follow.FilenameLineFollower(
        filename, tail_only=True
    ) as follower:
        for line in follower:
            line_queue.handle_line(prefix + line)


def tail_multiple(
    filenames,
    rate_limit=None,
    rate_period=None,
    each_rate_limit=None,
    each_rate_period=None,
):
    if TailHandler is not None:
        observer = Observer()
    # TODO: make configurable?
    prefix_start = "["
    prefix_end = "] "
    prefix_len = max(len(f) for f in filenames) + len(prefix_start) + len(prefix_end)
    threads = []
    stop = threading.Event()
    master = Master(stop, rate_limit, rate_period)
    master_thread = threading.Thread(target=master.run)
    master_thread.start()
    files = []
    try:
        for filename in filenames:
            prefix = "%s%s%s" % (prefix_start, filename, prefix_end)
            if len(prefix) < prefix_len:
                prefix += " " * (prefix_len - len(prefix))
            if each_rate_limit is None or each_rate_period is None:
                line_queue = master.line_queue
            else:
                line_queue = LineQueue(stop, each_rate_limit, each_rate_period)
                queue_chain = QueueChain(line_queue, master.line_queue, prefix + "...")
                chain_thread = threading.Thread(target=queue_chain.run)
                chain_thread.daemon = True
                chain_thread.start()
                threads.append(chain_thread)
            if filename == "-":
                thread = threading.Thread(
                    target=tail_file, args=(sys.stdin, line_queue, prefix)
                )
                thread.daemon = True
                thread.start()
                threads.append(thread)
            elif TailHandler is not None:
                if not os.path.exists(filename):
                    sys.stderr.write("file %s does not exist\n" % (filename,))
                    continue
                thefile = open(filename, "r")
                files.append(thefile)
                thefile.seek(0, 2)  # Go to the end of the file
                filename = os.path.abspath(filename)
                handler = TailHandler(filename, thefile, line_queue, prefix)
                observer.schedule(handler, path=os.path.dirname(filename))
            else:
                thread = threading.Thread(
                    target=tail_filename, args=(filename, line_queue, prefix)
                )
                thread.daemon = True
                thread.start()
                threads.append(thread)
        if observer is not None:
            observer.start()
        try:
            if threads:
                while threads:
                    for thread in threads:
                        thread.join(0.1)
                        if not thread.is_alive():
                            threads.remove(thread)
            else:
                while True:
                    time.sleep(0.1)
            stop.set()
            if observer is not None:
                observer.stop()
                observer.join()
            master_thread.join()
        except KeyboardInterrupt:
            sys.exit(0)
        finally:
            stop.set()
            if observer is not None:
                observer.stop()
                observer.join()
            master_thread.join()
    finally:
        for f in files:
            f.close()


def int_or_none(val):
    return None if val is None or val == "" else int(val)


def main():
    args = docopt(__doc__)
    if not args["--no-force-line-buffer"]:
        if hasattr(sys.stdout, "fileno"):
            sys.stdout = os.fdopen(sys.stdout.fileno(), "w", 1)
    rate_limit = int_or_none(args["--rate-limit"])
    rate_period = int_or_none(args["--rate-period"])
    each_rate_limit = int_or_none(args["--each-rate-limit"])
    each_rate_period = int_or_none(args["--each-rate-period"])
    tail_multiple(
        args["<filename>"],
        rate_limit=rate_limit,
        rate_period=rate_period,
        each_rate_limit=each_rate_limit,
        each_rate_period=each_rate_period,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Streams

Usage:
    stream.py -h | --help
    stream.py [--tail] [--byte [--bufsize=<bufsize>]] <filename>

Options:
    -h --help           Help.
    --tail              Only stream lines from the end of the file.
    --byte              Don't buffer, output new bytes as soon as they are
                        read.
    --bufsize=<bufsize> The number of bytes that will be attempted to be read
                        at a time. [Default: 1024]
"""
from __future__ import print_function
import os
import sys

from docopt import docopt

from reversefold.util import follow


def main():
    args = docopt(__doc__)
    if args["--byte"]:
        if hasattr(sys.stdout, "fileno"):
            # Force stdout to be un-buffered
            sys.stdout = os.fdopen(sys.stdout.fileno(), "w", 0)

        with follow.Follower(
            args["<filename>"], args["--tail"], int(args["--bufsize"])
        ) as follower:
            try:
                for data in follower:
                    sys.stdout.write(data)
                    sys.stdout.flush()
            except KeyboardInterrupt:
                follower.finish = True
                for data in follower:
                    sys.stdout.write(data)
                sys.stdout.flush()
                raise

    else:
        with follow.FilenameLineFollower(
            args["<filename>"], args["--tail"]
        ) as follower:
            try:
                for data in follower:
                    print(data)
            except KeyboardInterrupt:
                follower.finish = True
                for data in follower:
                    print(data)
                raise


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['reversefold', 'reversefold.util']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.1,<0.5.0',
 'docopt>=0.6.2,<0.7.0',
 'fasteners>=0.15.0,<0.16.0',
 'psutil>=5.6,<6.0',
 'python-daemon>=2.1,<3.0',
 'watchdog>=0.9.0,<0.10.0']

entry_points = \
{'console_scripts': ['daemonize.py = reversefold.util.daemonize:main',
                     'log.py = reversefold.util.log:main',
                     'sort_json.py = reversefold.util.sort_json:main',
                     'stream.py = reversefold.util.stream:main',
                     'tail.py = reversefold.util.tail:main']}

setup_kwargs = {
    'name': 'reversefold.util',
    'version': '3.5.5',
    'description': 'SSH, Proc, Multiproc, tail.py, log.py, stream.py, daemonize.py, etc.',
    'long_description': "# reversefold.util\n\n[Available on pypi](https://pypi.python.org/pypi/reversefold.util)\n\nThis is a collection of various command-line scripts and libraries which have come in useful over the years I've worked with Python.\n\n\n## log.py\n\nCaptures stdout of a process and enables transformation the output (such as adding a timestamp to each line) and is compatible with external logrotate through `WatchedFileHandler`.\n\n## daemonize.py\n\nUseful for daemonizing another process which either does not daemonize itself or for which you want to capture stdout and stderr to log files. Uses `WatchedFileHandler` for output to log files to allow for external log rotation.\n\n## stream.py\n\nSimilar to `tail -f` but with some more options for type of buffering and supports streaming the entire current contents of the file before then following the tail of the file.\n\n## reversefold.util\n\n### rate_limit_gen\n\nA generator wrapper which rate-limits another generator. If the rate is exceeded, further values received within the `period` are discarded. Useful, for example, for making sure that the number of lines you display from a log file you're following don't cause your terminal to block while displaying a huge amount of output.\n\n### chunked\n\nBreaks up an iterable into equal-sized chunks.\n\n## reversefold.util.ssh\n\n### SSHHost\n\nA programmatic interface to ssh. Allows easily running a single command or a shell script or interactively sending input and displaying output. Originally written as a drop-in monkeypatch for fabric's use of paramiko.\n\n## reversefold.util.multiproc\n\n### run_subproc\n\nTakes a subprocess as input and sets up threads for handling and displaying stdout and stderr from the process. Defaults to blocking until the process is finished but also supports immediately returning and including the threads in the return value. Also defaults to capturing the stdout and stderr and returning them as lists of lines.\n\n## reversefold.util.proc\n\nProvides context managers to ensure that a process is sent a TERM or KILL signal (or both) to a process when the context block exits. Can optionally find child processes recursively and send the same signal(s) to them. Also provides functions for the same functionality.\n",
    'author': 'Justin Patrin',
    'author_email': 'papercrane@reversefold.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/reversefold/util',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>3.4,<4',
}


setup(**setup_kwargs)



#!/usr/bin/env python
"""Make a process which runs in the foreground run detached and capture its output.

This script will exit when the command exits.

Usage:
    %(script)s [--stdout-log=<stdout-log>] [--stderr-log=<stderr-log>]
               [--pidfile=<pidfile>] [--app-pidfile=<pidfile>]
               [--log-format=<fmt> [--date-format=<fmt>]]
               [(--log-handler=watched | --log-handler=timed [--when=<w>] [--interval=<i>] [--backup-count=<b>])]
               [--debug]
               -- <command>...

Options:
    -h --help                     Show this help text.
    -p --pidfile=<pidfile>        Path to a pidfile which will hold this script's pid (not the underlying process).
    -d --app-pidfile=<pidfile>    Path to a pidfile which will hold the pid of <command>.
    -o --stdout-log=<stdout-log>  Path to log which will hold the stdout of the command [Default: log/stdout.log]
    -e --stderr-log=<stderr-log>  Path to log which will hold the stderr of the command [Default: log/stderr.log]
                                  The special value STDOUT will put this in the same log as the stdout output.
    --log-format=<fmt>            The format that will be applied to the stdout and stderr logs. [Default: %%(message)s]
                                  For example, if you wanted to prepend a timestamp to each line you could use:
                                  %%(asctime)s %%(message)s
                                  For a full reference see the Python documentation:
                                  https://docs.python.org/2/library/logging.html#logrecord-attributes
    --date-format=<fmt>           The format to apply to the logging timestamp [Default: %%Y-%%m-%%d %%H:%%M:%%S]
                                  Note that this option won't take effect unless --log-format is given with a date
                                  identifier (like %%(asctime)s) in it.
                                  For a full reference see the Python documentation:
                                  https://docs.python.org/2/library/datetime.html#strftime-strptime-behavior
    --log-handler=<type>          Set the log handler type. [Default: watched]
                                  Options:
                                    "watched": WatchedFileHandler. If the log is rotated via an outside process
                                      such as logrotate this handler will detect that and create a new file with the
                                      original name.
                                    "timed": TimedRotatingFileHandler. Will automatically rotate the log file at time
                                      intervals.
    --when=<w>                    Sets the when parameter of the TimedRotatingFileHandler. [Default: h]
                                  Valid options:
                                    s (second)
                                    m (minute)
                                    h (hour)
                                    d (day)
                                    w0-w6 (weekday, 0=Monday)
                                    midnight
    --interval=<i>                Sets the interval parameter of the TimedRotatingFileHandler. [Default: 1]
    --backup-count=<b>            Sets the backupCount parameter of the TimedRotatingFileHandler. [Default: 24]
    --debug                       More output.
"""
from __future__ import print_function
from datetime import datetime, timedelta
import functools
import io
import logging
import logging.handlers
import os
import subprocess
import sys
import threading
import time

import daemon
from docopt import docopt
from fasteners import process_lock
import psutil


from reversefold.util import multiproc


LOG = logging.getLogger(__name__)


class Error(Exception):
    pass


def get_logger(
    name, filename, log_format, date_format, log_handler, when, interval, backup_count
):

    if log_handler == "watched":
        handler = logging.handlers.WatchedFileHandler(filename)
    elif log_handler == "timed":
        handler = logging.handlers.TimedRotatingFileHandler(
            filename, when=when, interval=interval, backupCount=backup_count
        )
    formatter = logging.Formatter(log_format, datefmt=date_format)
    handler.setFormatter(formatter)

    handler.setLevel(logging.INFO)
    logger = logging.getLogger("daemonize.%s" % (name,))
    logger.setLevel(logging.INFO)
    logger.addHandler(handler)
    return logger, handler


class LockedPidFile(object):
    def __init__(self, pidfile_path):
        self.pidfile_path = pidfile_path
        self.pidfile_lock_path = self.pidfile_path + ".lock"
        self.pidfile = None
        self.pidfile_lock = None

    def acquire(self, pid=None):
        self.pidfile_lock = process_lock.InterProcessLock(self.pidfile_lock_path)
        if not self.pidfile_lock.acquire():
            LOG.error("Unable to acquire pifile_lock %s", self.pidfile_lock_path)
            return False
        if pid is None:
            pid = os.getpid()
        if os.path.exists(self.pidfile_path):
            try:
                with open(self.pidfile_path) as f:
                    stalepid = int(f.read())
                if stalepid in psutil.pids():
                    raise Error(
                        "Pidfile %s exists and is not locked but pid %s is still alive"
                        % (self.pidfile_path, stalepid)
                    )
            except Exception as exc:
                LOG.exception("Exception %r checking the stale pidfile, ignoring", exc)
            LOG.warning(
                "Pidfile exists but is not locked, removing %s", self.pidfile_path
            )
            os.unlink(self.pidfile_path)

        self.pidfile = open(
            self.pidfile_path, "x", opener=functools.partial(os.open, mode=0o600)
        )
        self.pidfile.write(str(pid))
        self.pidfile.flush()
        return True

    def release(self):
        if not self.pidfile_lock.acquired:
            raise Error("Lock is not locked")
        self.pidfile.close()
        if os.path.exists(self.pidfile_path):
            os.unlink(self.pidfile_path)
        os.unlink(self.pidfile_lock_path)
        self.pidfile_lock.release()

    def __enter__(self):
        acquired = self.acquire()
        if acquired:
            return self.pidfile
        raise Error("Could not acquire lock")

    def __exit__(self, exc_type, exc_value, traceback):
        self.release()


def main():
    docstr = __doc__ % {"script": os.path.basename(__file__)}
    args = docopt(docstr)
    if args["--when"] not in (
        "s",
        "m",
        "h",
        "d",
        "w0",
        "w1",
        "w2",
        "w3",
        "w4",
        "w5",
        "w6",
        "midnight",
    ):
        sys.stderr.write(
            "Value %r for --when parameter not understood.\n" % (args["--when"],)
        )
        sys.stderr.write(docstr)
        sys.exit(1)
    if args["--debug"]:
        print(args)
    os.umask(0o077)
    out_logger, out_handler = get_logger(
        "stdout",
        args["--stdout-log"],
        args["--log-format"],
        args["--date-format"],
        args["--log-handler"],
        args["--when"],
        int(args["--interval"]),
        int(args["--backup-count"]),
    )
    preserve = [out_handler.stream]
    if args["--stderr-log"] == "STDOUT":
        err_logger = out_logger
        err_handler = out_handler
    else:
        err_logger, err_handler = get_logger(
            "stderr",
            args["--stderr-log"],
            args["--log-format"],
            args["--date-format"],
            args["--log-handler"],
            args["--when"],
            int(args["--interval"]),
            int(args["--backup-count"]),
        )
        preserve.append(err_handler.stream)

    if args["--pidfile"]:
        pidfile = LockedPidFile(args["--pidfile"])
    else:
        pidfile = None

    if args["--app-pidfile"]:
        applock = LockedPidFile(args["--app-pidfile"])
    else:
        applock = None

    # DaemonContext is about to close all of our file handles so our logger will stop working. Add the error handler
    # to our logger so any logging after this won't be lost.
    LOG.addHandler(err_handler)
    with daemon.DaemonContext(
        pidfile=pidfile, working_directory=os.getcwd(), files_preserve=preserve
    ):
        try:
            proc = subprocess.Popen(
                args["<command>"],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
                if args["--stderr-log"] == "STDOUT"
                else subprocess.PIPE,
            )
            if applock and not applock.acquire(proc.pid):
                try:
                    proc.terminate()
                    start = datetime.datetime.now()
                    while proc.poll() is None and datetime.datetime.now() - start < datetime.timedelta(
                        seconds=1
                    ):
                        time.sleep(0.1)
                    if proc.poll() is None:
                        proc.kill()
                except Exception:
                    LOG.exception(
                        "Exception killing the app process after acquiring the pidfile failed."
                    )
                raise Error(
                    "Could not acquire app pidfile lock %s" % (applock.file_path,)
                )
            proc.stdin.close()
            thread = threading.Thread(
                target=multiproc.LinePipe(
                    "",
                    io.TextIOWrapper(proc.stdout, errors="backslashreplace"),
                    None,
                    out_logger.info,
                    "",
                    False,
                ).flow
            )
            thread.start()
            threads = [thread]
            if args["--stderr-log"] != "STDOUT":
                thread = threading.Thread(
                    target=multiproc.LinePipe(
                        "",
                        io.TextIOWrapper(proc.stderr, errors="backslashreplace"),
                        None,
                        err_logger.info,
                        "",
                        False,
                    ).flow
                )
                thread.start()
                threads.append(thread)
            while proc.returncode is None:
                time.sleep(0.1)
                proc.poll()
            start = datetime.now()
            wait_time = timedelta(seconds=5)
            while threads and datetime.now() - start < wait_time:
                alive_threads = []
                for thread in threads:
                    thread.join(timeout=0.1)
                    if thread.is_alive():
                        alive_threads.append(thread)
                threads = alive_threads
        except Exception:
            LOG.exception("Exception within DaemonContext")
        finally:
            if applock:
                applock.release()
        sys.exit()


if __name__ == "__main__":
    main()



import contextlib
import logging
import os
import pwd
import signal

import psutil


LOG = logging.getLogger(__name__)

_SENTINEL = object()


# TODO: support arbitrary signals, not just TERM and KILL.
# Potentially support lists of signals (with wait times for graceful shutdown?).


def _signal_processes(procs, sig):
    for proc in procs:
        if proc.is_running():
            try:
                proc.send_signal(sig)
            except psutil.NoSuchProcess:
                pass
            except Exception as e:
                LOG.error("Exception sending signal %s to %r: %r", sig, proc, e)


def get_process_tree(pid):
    try:
        proc = psutil.Process(pid)
    except psutil.NoSuchProcess:
        return []
    return [proc] + proc.children(recursive=True)


# Supports any process object with a pid attribute, so supports at least psutil.Process and subprocess.Popen objects.
@contextlib.contextmanager
def signalling(proc, sig, recursive=False, _procs=None):
    exc = False
    try:
        yield proc
    except:  # noqa
        exc = True
        raise
    finally:
        try:
            if recursive:
                procs = get_process_tree(proc.pid)
                # if _procs was passed in, add our processes to the list and use the combined list for signalling below
                if _procs is not None:
                    for child in procs:
                        _procs.add(child)
                    procs = list(_procs)
            else:
                try:
                    procs = [psutil.Process(proc.pid)]
                except psutil.NoSuchProcess:
                    procs = []
            _signal_processes(procs, sig)
        except Exception as e:
            if exc:
                LOG.exception("Exception sending signal %s to %r: %r", sig, procs, e)
            else:
                raise


def interrupting(proc, recursive=False, _procs=None):
    return signalling(proc, signal.SIGINT, recursive=recursive, _procs=_procs)


def terminating(proc, recursive=False, _procs=None):
    return signalling(proc, signal.SIGTERM, recursive=recursive, _procs=_procs)


def killing(proc, recursive=False, _procs=None):
    return signalling(proc, signal.SIGKILL, recursive=recursive, _procs=_procs)


@contextlib.contextmanager
def dead(proc, recursive=False):
    if recursive:
        # This object is passed into the killing and terminating contextmanagers, allowing them to pass the list of
        # processes they are working on between them so that if the parent process dies in the terminating context
        # manager but a child process does not die, it will still get the kill signal from the killing contextmanager.
        procs = set()
    else:
        procs = None
    with killing(proc, recursive=recursive, _procs=procs) as kproc:
        with terminating(kproc, recursive=recursive, _procs=procs) as tproc:
            yield tproc


def interrupt(proc, recursive=False):
    with interrupting(proc, recursive=recursive):
        pass


def terminate(proc, recursive=False):
    with terminating(proc, recursive=recursive):
        pass


def kill(proc, recursive=False):
    with killing(proc, recursive=recursive):
        pass


def die(proc, recursive=False):
    with dead(proc, recursive=recursive):
        pass


def get_processes_in_path(path, owner=_SENTINEL):
    if owner is _SENTINEL:
        owner = pwd.getpwuid(os.getuid()).pw_name
    procs = []
    for proc in psutil.process_iter():
        if owner is not None and proc.username() != owner:
            continue
        try:
            if proc.exe().startswith(path) or any(
                f.path.startswith(path) for f in proc.open_files()
            ):
                procs.append(proc)
        except psutil.AccessDenied:
            pass
    return procs



#!/usr/bin/env python
import collections
import json
import sys


def recursive_sorted(data):
    if isinstance(data, dict):
        return collections.OrderedDict(
            sorted((k, recursive_sorted(v)) for k, v in data.iteritems())
        )
    elif isinstance(data, (list, tuple)):
        return sorted(recursive_sorted(v) for v in data)
    else:
        return data


def main():
    data = json.load(sys.stdin)
    print(json.dumps(recursive_sorted(data), indent=4))


if __name__ == "__main__":
    main()
